import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import org.apache.xerces.parsers.DOMParser;
import org.apache.xerces.dom.DocumentImpl;
import org.apache.xerces.dom.ElementImpl; 
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Attr;
import org.apache.xml.serialize.XMLSerializer;
import org.apache.xml.serialize.OutputFormat;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.stream.StreamResult;

public class ReviewOrder extends HttpServlet {

  public String getNodeTypeName(int type) {

      String[] nodeTypeNames = { "No node type 0",
                                 "ELEMENT_NODE",
                                 "ATTRIBUTE_NODE",
                                 "TEXT_NODE",
                                 "CDATA_SECTION_NODE",
                                 "ENTITY_REFERENCE_NODE",
                                 "ENTITY_NODE",
                                 "PROCESSING_INSTRUCTION_NODE",
                                 "COMMENT_NODE",
                                 "DOCUMENT_NODE",
                                 "DOCUMENT_TYPE_NODE",
                                 "DOCUMENT_FRAGMENT_NODE",
                                 "NOTATION_NODE" };
      return nodeTypeNames[type];
  }

  public void processNode(Node thisNode,
                          PrintWriter out,
                          Node newParent,
                          DocumentImpl newDoc,
                          String namespaceURI,
                          String ns)  {

      if (thisNode.getNodeType() == thisNode.ELEMENT_NODE) {

          Element newElement =
                  newDoc.createElementNS(namespaceURI, 
                                         ns+thisNode.getNodeName());
          newParent.appendChild(newElement);

          NamedNodeMap attributes = thisNode.getAttributes();
          if (attributes.getLength() > 0) {
              for (int i=0; i < attributes.getLength(); i++) {
                  Node thisAttr = attributes.item(i);
                  Attr thisAttrObj =
                       newDoc.createAttributeNS(namespaceURI,
                                   ns+thisAttr.getNodeName());
                  thisAttrObj.setValue(thisAttr.getNodeValue());
                  newElement.setAttributeNode(thisAttrObj);
              }
          }

          NodeList thisKids = thisNode.getChildNodes();
          for (int i=0; i < thisKids.getLength(); i++) {
              processNode(thisKids.item(i), out, newElement,
                                             newDoc,namespaceURI, ns);
          }

      } else if (thisNode.getNodeType() ==
                         thisNode.TEXT_NODE) {
          Node newNode =
                newDoc.createTextNode(thisNode.getNodeValue());
          newParent.appendChild(newNode);
      } else {
          Node nextNode = thisNode.getNextSibling();
          if (nextNode == null) {
               // The document is finished.
          } else {
               processNode(nextNode, out, newParent, 
                                          newDoc, namespaceURI, ns);
          }
      }
  }

  public void doGet(HttpServletRequest request,
                    HttpServletResponse response)
              throws IOException, ServletException
  {
      response.setContentType("text/html");
      PrintWriter out = response.getWriter();

      try {

          DOMParser parser = new DOMParser();
          parser.parse("/files/orders.xml");
          Document orders = parser.getDocument();

          DocumentImpl newDoc = new DocumentImpl();
          Element orderSystem = newDoc.createElement("orderSystem");
          orderSystem.setAttribute("xmlns:ord",
                           "http://www.nicholaschase.com/orders/");
          orderSystem.setAttribute("xmlns:prod",
                           "http://www.nicholaschase.com/products/");

          newDoc.appendChild(orderSystem);

          HttpSession session = request.getSession(true);
          String session_id = session.getId();

          NodeList allOrders = orders.getElementsByTagName("order");
          for (int i=0; i < allOrders.getLength(); i++) {
              NamedNodeMap attributes = allOrders.item(i).getAttributes();
              Node order_idNode = attributes.item(0);
              String order_id = order_idNode.getNodeValue();
              if (order_id.equals(session_id)) {
                  processNode(allOrders.item(i), out, orderSystem, newDoc,
                     "http://www.nicholaschase.com/orders/", "ord:");
              }
          }

          parser.parse("/files/products.xml");
          Document products = parser.getDocument();
          Node productRoot = products.getFirstChild();
          processNode(productRoot, out, orderSystem, newDoc, 
              "http://www.nicholaschase.com/products/", "prod:");

          TransformerFactory transFactory = 
               TransformerFactory.newInstance();

          Transformer transformer = transFactory.newTransformer
                  (new StreamSource("file:///c:/files/reviewOrder.xsl"));

          DOMResult domResult = new DOMResult();

          transformer.transform(new DOMSource(newDoc), domResult);

          Document tempDoc = new DocumentImpl();
          tempDoc = (Document)domResult.getNode();

          float totalCost = 0;
          float itemCost = 0;

          ElementImpl tempDocRoot = (ElementImpl)tempDoc.getFirstChild();
          NodeList items = tempDocRoot.getElementsByTagName("ord:item");
          for (int i=0; i < items.getLength(); i++) {
              ElementImpl thisItem = (ElementImpl)items.item(i);

              NodeList quantityNodeList = 
                           thisItem.getElementsByTagName("ord:quantity");
              Node quantityNode=quantityNodeList.item(0).getFirstChild();

              int quantity=Integer.parseInt(quantityNode.getNodeValue());

              NodeList priceNodeList = 
                         thisItem.getElementsByTagName("prod:price");
              Node priceNode = priceNodeList.item(0).getFirstChild();

              String priceString = priceNode.getNodeValue().substring(1);
              float price = Float.valueOf(priceString).floatValue();

              itemCost = quantity * price;

              Node itemCostNode = tempDoc.createElement("ord:itemCost");
              Node itemCostText = 
                       tempDoc.createTextNode(String.valueOf(itemCost));

              itemCostNode.appendChild(itemCostText);
              thisItem.appendChild(itemCostNode);
              totalCost = totalCost + itemCost;
          }

          Node totalCostNode = tempDoc.createElement("ord:totalCost");
          Node totalCostText = 
                    tempDoc.createTextNode(String.valueOf(totalCost));

          totalCostNode.appendChild(totalCostText);
          tempDocRoot.appendChild(totalCostNode);

          transformer = transFactory.newTransformer
                  (new StreamSource("file:///c:/files/confirmOrder.xsl"));
          transformer.transform(new DOMSource(tempDoc), 
                                new StreamResult(out));

      } catch (org.xml.sax.SAXException SAXe) {
          out.print("There was a problem with the source"
              + " document.");
      } catch (IOException IOe) {
          out.print("There was an IO error. Make sure the file"
              + " exists and is readable.");
      } catch (Exception e) {
          out.print("There was an error: "+e.getMessage());
      }

  }
}

