<?xml version="1.0"?>
<xsl:stylesheet version="1.0" 
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:fo="http://www.w3.org/1999/XSL/Format">

<xsl:template match="products">
<fo:root>
    <fo:layout-master-set>

        <fo:simple-page-master master-name="first"
                  page-height="11in" 
                  page-width="8.5in"
                  margin-top="1in" 
                  margin-bottom=".75in" 
                  margin-left="1in" 
                  margin-right="1in">
            <fo:region-body margin-top=".75in" />
        </fo:simple-page-master>    

        <fo:simple-page-master master-name="rest"
                  page-height="11in" 
                  page-width="8.5in"
                  margin-top="1in" 
                  margin-bottom=".75in" 
                  margin-left="1in" 
                  margin-right="1in">
            <fo:region-body margin-top=".75in"/>
            <fo:region-before extent=".5in"/>
            <fo:region-after extent=".75in"/>
        </fo:simple-page-master>

        <fo:page-sequence-master master-name="layout" >
            <fo:repeatable-page-master-alternatives>
                <fo:conditional-page-master-reference 
                      master-name="first" 
                      page-position="first" />
                <fo:conditional-page-master-reference 
                      master-name="rest"
                      page-position="rest" />

                <fo:conditional-page-master-reference 
                      master-name="rest" />
            </fo:repeatable-page-master-alternatives>
        </fo:page-sequence-master>

    </fo:layout-master-set>
 
    <fo:page-sequence master-name="layout">

        <fo:static-content flow-name="xsl-region-before">
            <fo:block text-align="end" 
                font-size="10pt" 
                font-family="sans-serif">
               
                ChaseWeb Spring Catalog

            </fo:block>
        </fo:static-content> 

        <fo:flow flow-name="xsl-region-body">

            <fo:block text-align="center">
                <fo:external-graphic 
                   src="file:///c:/files/chasewebfurniture.gif"/>
            </fo:block>

            <fo:block font-size="18pt" 
                font-family="sans-serif" 
                background-color="red"
                color="yellow"
                text-align="center"
                padding-top="3pt">

                The ChaseWeb Furniture Spring Catalog
            </fo:block>
 
            <fo:block font-size="24pt" 
                font-family="sans-serif" 
                space-before.optimum=".25in"
                space-before.minimum=".1in"
                space-before.maximum=".5in"
                text-align="center">
 
                Table of Contents

            </fo:block>
            <xsl:for-each select="vendor">
           
                <fo:block font-size="18pt" 
                     font-family="sans-serif" 
                     space-before.optimum=".25in"
                     space-before.minimum=".1in"
                     space-before.maximum=".5in"
                     text-align="start">
   
                     <xsl:value-of select="vendor_name" />
 
                </fo:block>

                <xsl:for-each select="product | suite">           

                    <fo:block font-size="12pt" 
                        font-family="sans-serif" 
                        text-align="start">

                        <xsl:value-of select="product_id" />
                        -- 
                        <xsl:value-of select="short_desc" />  
                    </fo:block>
                </xsl:for-each>

            </xsl:for-each>

            <xsl:for-each select="vendor">
                <xsl:call-template name="vendor_page" />
            </xsl:for-each>

        </fo:flow>

        <fo:static-content flow-name="xsl-region-after">
            <fo:block text-align="center" 
                font-size="10pt" 
                font-family="sans-serif">
                
                Page <fo:page-number/>

            </fo:block>
        </fo:static-content> 

    </fo:page-sequence>

</fo:root>

</xsl:template>

<xsl:template name="vendor_page">

    <fo:block break-before="page" 
        font-size="24pt"
        font-family="sans-serif" 
        text-align="center"
        font-weight="bold">

        <xsl:value-of select="vendor_name" />

    </fo:block>

    <fo:block font-size="14pt" 
        font-family="sans-serif" 
        space-before.optimum=".25in"
        text-align="start">

        <xsl:value-of select="advertisement" />
  
    </fo:block>

    <xsl:for-each select="product | suite">
        <xsl:call-template name="product_block" />
    </xsl:for-each>

</xsl:template>

<xsl:template name="product_block">

    <fo:block font-size="14pt" 
        font-family="sans-serif" 
        space-before.optimum=".25in"
        font-weight="bold"
        text-align="start">

        <xsl:value-of select="product_id" />
        --
        <xsl:value-of select="short_desc" />
        <xsl:value-of select="product_desc" />

    </fo:block>

    <fo:block font-size="12pt" 
        font-family="sans-serif" 
        space-before.optimum=".25in"
        text-align="start">

        Price:  <xsl:value-of select="price[@pricetype='retail']" />

    </fo:block>

    <xsl:if test="inventory[@location='showroom' and @color]">
        <fo:block font-size="12pt" 
            font-family="sans-serif" 
            space-before.optimum=".25in"
            text-align="start">
  
            Available in these great colors:
            <fo:list-block>    
                <xsl:for-each 
                    select="inventory[@location='showroom' and @color]">
                    <fo:list-item>
                        <fo:list-item-label>
                            <fo:block>&#x2022;</fo:block>
                        </fo:list-item-label>
                        <fo:list-item-body>
                            <fo:block>
                                <xsl:value-of select="@color"/>
                            </fo:block>
                        </fo:list-item-body>
                    </fo:list-item>
  
                </xsl:for-each>
            </fo:list-block>

        </fo:block>
    </xsl:if>

    <fo:block>
        <fo:leader leader-pattern="rule" 
            space-before.optimum="12pt" 
            space-after.optimum="12pt"/>
    </fo:block>

</xsl:template>

</xsl:stylesheet>
