import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import org.jdom.Element;
import org.jdom.Document;
import org.jdom.JDOMException;
import org.jdom.Namespace;
import org.jdom.output.XMLOutputter;
import org.jdom.input.SAXBuilder;
import java.util.List;
import org.jdom.output.DOMOutputter;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

public class AddToOrder extends HttpServlet {

  //*******BEGIN returnResult()*********
  public void returnResult( String product_id,
                            PrintWriter out )
  {
      File resultFile = new File("/files/resultText.xml");
      Document resultDoc = getDocument(resultFile, out);
      Element resultText = resultDoc.getRootElement();
      Element productIdHere = resultText.getChild("productIdHere");
      productIdHere.addContent(product_id);
      File includeFile = new File("/files/topinclude.xsl");
      Document includeDoc = getDocument(includeFile, out);
      Element styleSheetRoot = includeDoc.getRootElement();
 
      List templates = styleSheetRoot.getChildren();
      for (int i=0; i < templates.size(); i++) {
          Element thisTemplate = (Element)templates.get(i);
          String thisMatch = thisTemplate.getAttributeValue("match");
          if (thisMatch.equals("/")) {
              Element htmlElement = thisTemplate.getChild("html");
              Element bodyElement = htmlElement.getChild("body");
              Namespace xslns = Namespace.getNamespace("xsl",
                          "http://www.w3.org/1999/XSL/Transform");
              List allContent =
                   bodyElement.getChildren("apply-templates", xslns);
              allContent.set(0, resultText);
              break;
          }
      }

      try {
          org.w3c.dom.Document DOMDoc;
          DOMOutputter DomOut = new DOMOutputter();
          DOMDoc = DomOut.output(includeDoc);

          TransformerFactory transFactory = 
               TransformerFactory.newInstance();
          Transformer transformer = 
               transFactory.newTransformer(new DOMSource(DOMDoc));
          transformer.transform(new DOMSource(DOMDoc), 
                                new StreamResult(out));
      } catch (Exception e) {
          out.print("There was a problem with the transformation. ");
      }
  }
  //*******END returnResult()*********

  //*******BEGIN getDocument() *********
  public Document getDocument(File sourceFile,
                              PrintWriter errorsOut)
  {
    try {

        SAXBuilder builder = new SAXBuilder();
        Document document = builder.build(sourceFile);
        return document;

    } catch (JDOMException e) {
        errorsOut.print("There was a problem building the document: "
               +e.getMessage()+"<br />"+
               "Returning blank document.");
        return new Document(new Element("blank"));
    }
  }
  //*******END getDocument() *********

  //*******BEGIN saveDocument() *********
  public void saveDocument (Document saveDoc,
                            File saveFile,
                            PrintWriter errorsOut) {
      try {
          FileOutputStream outStream =
                      new FileOutputStream(saveFile);

          XMLOutputter outToFile =
                      new XMLOutputter("    ", true);
          outToFile.output(saveDoc, outStream);

          outStream.flush();
          outStream.close();
      } catch (IOException e) {
          errorsOut.print("Can't save orders.xml: "
                           + e.getMessage()+"<br />");
      }
  }
  //*******END saveDocument() *********

  //******BEGIN addItem() *********
  public void addItem(Element orderElement, 
                      String product_id, 
                      String quant) {

      Element newItem = new Element("item");
      newItem.addAttribute("product_id", product_id);
      Element quantity = new Element("quantity");
      quantity.addContent(quant);
      newItem.addContent(quantity);
      orderElement.addContent(newItem);

  }
  //*******END  addItem() *********

  public void doGet(HttpServletRequest request,
                    HttpServletResponse response)
            throws IOException, ServletException
  {
    // Retrieve item to be added
    String thisProduct_id =
             request.getParameter("addProduct_id");
    String thisQuantity =
             request.getParameter("addQuantity");

    response.setContentType("text/html");
    PrintWriter out = response.getWriter();
    
    // Does orders.xml exist?
    File orderFile = new File ("/files/orders.xml");

    // If orders.xml doesn't exist:
    if (!orderFile.exists()) {
        //     Create orders.xml
        Element root = new Element("orders");
        root.addContent(" ");
        Document document = new Document(root);

        //     Save orders.xml
        saveDocument (document, orderFile, out);
    } else {
        // Do Nothing
    }

    // Get existing orders from orders.xml
    Document document = getDocument(orderFile, out);

    // Does this order exist?
    HttpSession session = request.getSession(true);
    String session_id = session.getId();

    Element root = document.getRootElement();
    List orders = root.getChildren();
    int num_orders = orders.size();
    boolean orderExists = false;
    int orderIndex = 0;
    for (int i=0; i < num_orders; i++) {
        Element iOrder = (Element)orders.get(i);
        String iOrder_id = iOrder.getAttributeValue("order_id");
        if (iOrder_id.equals(session_id)) {
            orderExists = true;
            orderIndex = i;
            break;
        }
    }

    // If this order does not exist:
    if (!orderExists) {
        //     Create this order

        Element order = new Element("order");
        order.addAttribute("order_id", session_id);
        Element status = new Element("order_status");
        status.setText("in progress");
        order.addContent(status);

        // If the order didn't exist, the item certainly
        // wasn't part of it.
        addItem(order, thisProduct_id, thisQuantity);
        root.addContent(order);
    }

    Element thisOrder = (Element)orders.get(orderIndex);
    boolean itemExists = false;
    int itemIndex = 0;
    List items = thisOrder.getChildren("item");
    int num_items = items.size();
    for (int i=0; i < num_items; i++) {
        Element iItem = (Element)items.get(i);
        String iProduct_id =
                   iItem.getAttribute("product_id").getValue();
        if (iProduct_id.equals(thisProduct_id)) {
          itemExists = true;
          itemIndex = i;
          break;
       }
    }

    // If this item is already part of this order:
    if (itemExists) {
        //     Increment the quantity
        Element thisItem = (Element)items.get(itemIndex);
        String currentQuantity =
                            thisItem.getChildText("quantity");
        int newQuantity = Integer.parseInt(currentQuantity)
                           + Integer.parseInt(thisQuantity);

        String strQuantity = new String().valueOf(newQuantity);
        thisItem.getChild("quantity").setText(strQuantity);
    } else {
        //     Add this item to this order
        addItem(thisOrder, thisProduct_id, thisQuantity);
    }

    // Save orders.xml
    saveDocument (document, orderFile, out);

    returnResult (thisProduct_id, out);
  
  }
  //*******END doGet() *********

  //*******BEGIN doPost() *********
  public void doPost(HttpServletRequest request,
                      HttpServletResponse response)
             throws IOException, ServletException
  {
      doGet(request, response);
  }
 //*******END doPost() *********
}

